/**********************************************************************************
   Copyright(c) Catalyst Semiconductor, Inc., 2005. All rights reserved.

This code is provided as is without warranty or guarantee of any kind as its accuracy, 
completeness, operability, fitness for particular purpose, or any other warranty.

   File        :  SPImaster.v
   Description :  Verilog model for SPI protocol - master side
		  for testing Verilog model for CAT25XXXX SPI serial EEPROMs
   Version     :  Rev1.00 
   Release     :  Jan, 2005
**********************************************************************************/

 `timescale 1ns/1ns
//
//	Master for SPI protocol
//
module SPImaster(SI,SO,SCK,CSB,WPTST,WP,HOLD,HOLDS);
	output 		SI,SCK,CSB,WP,HOLDS;
	reg SI,SCK,CSB,WP,HOLDS;
	input SO,HOLD,WPTST;
	tri SO;
//
`include "MemType.v"
//
`define B0			1'b0
`define B1			1'b1
//
//	states
//
`define idle 			4'b0000
`define sendingInstruction 	4'b0001
`define sendingAdd 		4'b0010
`define sendingData		4'b0011
`define sendingStopStart	4'b0100
`define receivingData		4'b0111
`define holdS			4'b1111
//
//	instructions
//
`define EnableWrite 		1'b000
`define DisableWrite 		1'b001
`define ReadStatus 		1'b010
`define WriteStatus 		1'b011
`define ReadData 		1'b100
`define WriteData 		1'b101
//
parameter mode = 2'b00;
parameter frequency = 1000000; //(Hz)

//
// Read timing
//

	`ifdef cat25c01
		`include "Tim_1_2_4_8_16.v"
	`endif
	`ifdef cat25010
		`include "Tim_10_20_40.v"
	`endif

	`ifdef cat25c02
		`include "Tim_1_2_4_8_16.v"
	`endif
	`ifdef cat25020
		`include "Tim_10_20_40.v"
	`endif

	`ifdef cat25c04
		`include "Tim_1_2_4_8_16.v"
	`endif
	`ifdef cat25040
		`include "Tim_10_20_40.v"
	`endif
	`ifdef cat25c08
		`include "Tim_1_2_4_8_16.v"
	`endif
	`ifdef cat25c16
		`include "Tim_1_2_4_8_16.v"
	`endif
	`ifdef cat25c32
		`include "Tim_32_64.v"
	`endif
	`ifdef cat25c64
		`include "Tim_32_64.v"
	`endif
	`ifdef cat25c128
		`include "Tim_128_256.v"
	`endif
	`ifdef cat25c256
		`include "Tim_128_256.v"
	`endif

parameter delta = 10;

integer lSeq;						// sequence length 
integer MbitCounter, seqCounter, addrBytesCounter, i, j, recDataCounter, lgDisp;
integer tNORM, semForever, sem, seqLength;
integer TestPhase,addrBytes,addrBits;

integer tHALF;

reg [7:0] WREN, WRDI, RDSR, WRSR, READ, WRITE, WRONG_OPCODE;
reg[7:0] Minstruction, prevInstruction, manStReg;	// instruction
reg [3:0] Mstate,MnextState,savState,savStH,prevState; 	// State machine controller
reg bitA8;

reg [7:0] bufWriteData[0:4095];		
reg [7:0] bufReadData[0:4095];
reg [7:0] data[0:100];					// for data to write 
reg [7:0] linestr[0:79];				// for display

integer addr,addrMax,crtMemAddr,realCrtMemAddr; 

reg[7:0] pattern;
reg[7:0] genDataCell;
reg [7:0] MtempReg,MdataReg,manRegB; 
integer manRegW;
reg[6:0] manNr;
integer manAddr;
integer man0,man;
integer semPrepBegHold,semPrepEndHold;
event reinitClockForStart,go;

//
//#################################### TASKS ###########################################
//

task createIncremFillPattern;
	begin
	pattern = data[0];
	genDataCell = {pattern[7:4],4'b0000};
	for(i=0;i<lSeq;i=i+1)
		begin
		bufWriteData[i] = genDataCell;
			genDataCell = genDataCell+1;
		end
	end
endtask

task displayData;
	begin
	$display;
	$display("            MASTER: %d bytes of sent data: (addr = %h)",lSeq,addr);
	if(seqLength == 1)
		$display("----------");
	else
		$display("---------------------------------------------------------------------");
	for(i=0;i<seqLength+1;i=i+1)
		begin
		j=i%16;
		if(j == 0 && i>0)
			begin
			manAddr = addr+i-16;
			$display("%h:  %h  %h  %h  %h  %h  %h  %h  %h  %h  %h  %h  %h  %h  %h  %h  %h",manAddr,linestr[0],linestr[1],linestr[2],linestr[3],linestr[4],linestr[5],linestr[6],linestr[7],linestr[8],linestr[9],linestr[10],linestr[11],linestr[12],linestr[13],linestr[14],linestr[15]);
			linestr[j] = bufWriteData[i];
			end
		else
			linestr[j] = bufWriteData[i];
		end
	if(j>0)
		begin
		case(j)
		1:
			begin
			manAddr = addr+i-j-1;
			$display("%h:   %h",manAddr,linestr[0]);
			end
		default:
			i=i;	
		endcase
		end
	if(seqLength == 1)
		$display("----------");
	else
		$display("---------------------------------------------------------------------");
	end
endtask

task initBufReadData;
	begin
	for(i=0;i<4096;i=i+1)
		bufReadData[i] = 8'hff;	
	end
endtask

task displayRecData;
	begin
	if(Minstruction != RDSR)
		begin
		$display;
		$display("            MASTER: %d bytes of rec data: (addr = %h)",lSeq,addr);
		end
	if(recDataCounter == 1)
		begin
		if(Minstruction != RDSR)
			$display("----------");
		lgDisp = 1;
		end
	else
		begin
		$display("---------------------------------------------------------------------");
		lgDisp = recDataCounter;
		end
	for(i=0;i<lgDisp+1;i=i+1)
		begin
		j=i%16;
		if(j == 0 && i>0)
			begin
			manAddr = addr+i-16;
			$display("%h:  %h  %h  %h  %h  %h  %h  %h  %h  %h  %h  %h  %h  %h  %h  %h  %h",manAddr,linestr[0],linestr[1],linestr[2],linestr[3],linestr[4],linestr[5],linestr[6],linestr[7],linestr[8],linestr[9],linestr[10],linestr[11],linestr[12],linestr[13],linestr[14],linestr[15]);
			linestr[j] = bufReadData[i];
			end
		else
			linestr[j] = bufReadData[i];
		end
	if(j>0)
		begin
		case(j)
		1:
			begin
			manAddr = addr+i-j-1;

			if(Minstruction != RDSR)
				$display("%h:   %h",manAddr,linestr[0]);
			end	
		default:
			i=i;
		endcase
		end
	if(recDataCounter == 1)
		begin
		if(Minstruction != RDSR)
			$display("----------");
		else
			begin
			manStReg = linestr[0];
			end
		end
	else
		$display("---------------------------------------------------------------------");
	manRegW =  $time;
	end
endtask


//
//########################################## INITIAL #######################################
//
initial
	begin

	tHALF = 1000000000/(2*frequency);      // Half Clock Period (in ns)

	WREN =  8'b00000110;		// Enable Write
	WRDI =  8'b00000100;		// Disable Write
	RDSR =  8'b00000101;		// Read Status Register
	WRSR =  8'b00000001;		// Write Status Register
	READ =  8'b00000011;		// Read Data from Memory
	WRITE = 8'b00000010;		// Write Data to Memory
	WRONG_OPCODE =  8'b10000000;	// wrong operation code (for test)

	addr = 16'h0000;
	crtMemAddr = 16'h0000;
	realCrtMemAddr = 16'h0000;

	$display(" ");
	`ifdef cat25c01
		addrBytes = 1;
		addrBits = 7;
	`endif
	`ifdef cat25010
		addrBytes = 1;
		addrBits = 7;
	`endif

	`ifdef cat25c02
		addrBytes = 1;
		addrBits = 8;
	`endif
	`ifdef cat25020
		addrBytes = 1;
		addrBits = 8;
	`endif

	`ifdef cat25c04
		addrBytes = 1;
		addrBits = 9;
	`endif
	`ifdef cat25040
		addrBytes = 1;
		addrBits = 9;
	`endif

	`ifdef cat25c08
		addrBytes = 2;
		addrBits = 10;
	`endif
	`ifdef cat25c16
		addrBytes = 2;
		addrBits = 11;
	`endif
	`ifdef cat25c32
		addrBytes = 2;
		addrBits = 12;
	`endif
	`ifdef cat25c64
		addrBytes = 2;
		addrBits = 13;
	`endif
	`ifdef cat25c128
		addrBytes = 2;
		addrBits = 14;
	`endif
	`ifdef cat25c256
		addrBytes = 2;
		addrBits = 15;
	`endif
	//
	addrMax = 16'h0000;
	for(i=0; i<addrBits; i=i+1)
		addrMax[i]=1;	
	//
	TestPhase = 1;
	$display;
	$display("*******************************************************");
	$display("*		Master for SPI protocol");
	$display("*		SPI mode: %b",mode);
	$display("*******************************************************");
	Mstate = `idle;
	MnextState = `idle;
	MbitCounter = 0;
	seqCounter = lSeq;
	manStReg = 8'h00;
	prevInstruction = READ;
	SI = 1'b0;
	CSB = `B1;
	semForever = 0;
	sem = 0;
	MnextState = `sendingStopStart;
	SCK = 0;

	semPrepBegHold = 0;
	semPrepEndHold = 0;
	HOLDS = 1;

	man0 = frequency;
	man = 2*tHALF;
	$display("* Tclock  = %f ns (Frequency: %f Hz)",man,man0);
	$display("*");
/***********
	#0 if(man0 > 1000000*fMAX)
		begin		$display("*===============================================================================");
		$display("*\t WARNING: USED CLOCK FREQUENCY IS HIGHER THAN THE ALLOWED MAXIMUM ONE !");
		$display("*\t PLEASE, DO MODIFY THE CLOCK FREQUENCY.");
		$display("*===============================================================================");
		$display;
		$stop;
		end
*******/
	end

//
//############################## ALWAYS ####################################
//

always @(reinitClockForStart)
	begin
	while(semForever == 0)
		#tHALF  
			SCK = ~SCK;
	end

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
always @(posedge HOLD)
	begin
	if(SCK == 1'b0 && $time != 0)
		begin
		$display("MASTER: Preparing HOLD state ending...	",$time);
		Mstate = savStH;
		MnextState = savStH;
		if(MbitCounter == 8)
			begin
			MbitCounter = 0;
			MnextState = MnextState+1;
			end
		semPrepEndHold = 1;
		HOLDS = 1;
		semPrepEndHold = 0;
		end
	end

//------------------------------------------------------------------------------
always @(negedge HOLD) 	
	begin
	if(SCK == 1'b0 && $time != 0)
		begin
		$display("MASTER: Preparing HOLD state beginning ...",$time);
		savStH = Mstate;
		MnextState = `holdS;
		Mstate = `holdS;
		semPrepBegHold = 1;
		HOLDS = 0;
		semPrepBegHold = 0;
		end
	end

//+++++++++++++++++++++++++---------------
always @(WPTST)
	begin
	WP=WPTST;
	end

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ 00/11
//
//	State machine for RECEIVE type MASTER activity
// 
//---------------------------------------------------------------------- 01/10
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
always @(posedge SCK)
	begin
	if(mode==2'b00 || mode==2'b11)
     begin
	case(Mstate)
		`receivingData:		// 00 & 11
			begin
			#(tHALF/2) case(MbitCounter)
			1:
				begin
				MtempReg[7:0] = 8'b00000000;
				#0 MtempReg[0] = SO;
				#tHALF if(MbitCounter < 8 && MbitCounter != 0) 
					begin
					MbitCounter = MbitCounter + 1;
					end
				end
			8:
				begin
				MtempReg = MtempReg << 1;
				MtempReg[0] = SO;
				seqCounter = seqCounter - 1;
				if(seqCounter == 0)
					begin
					bufReadData[recDataCounter] = MtempReg[7:0];
					recDataCounter = recDataCounter + 1;
					crtMemAddr = crtMemAddr + 1;	//(inc.adr.in pag)
					realCrtMemAddr = crtMemAddr;
					if(recDataCounter == lSeq)
 						displayRecData;
					MnextState = `sendingStopStart;
					semForever = 1;
					/// #(tHALF/2) MbitCounter = 0;
					#(tHALF/4) MbitCounter = 0;
					// $display("            MASTER: rec. data %d: 		%h",lSeq-seqCounter,MtempReg[7:0]);
					if(Minstruction == RDSR)
						begin
						manStReg[7:0] = MtempReg[7:0];
						end
					/// #(tHALF/2) if(MbitCounter < 8 && MbitCounter != 0) 
					#(tHALF/4) if(MbitCounter < 8 && MbitCounter != 0) 
						begin
						MbitCounter = MbitCounter + 1;
						end
					end
				else
					begin
					bufReadData[recDataCounter] = MtempReg[7:0];
					recDataCounter = recDataCounter + 1;
					crtMemAddr = crtMemAddr + 1;	//(inc.adr.in pag)
					realCrtMemAddr = crtMemAddr;
					if(recDataCounter == lSeq)
						displayRecData;
					MnextState = `receivingData;
					// $display("            MASTER: rec. data %d: 		%h",lSeq-seqCounter,MtempReg[7:0]);
					#(tHALF/2) MbitCounter = 1;
					end
				end
			// cases 2,3,4,5,6,7
			default:
				begin
				MtempReg = MtempReg << 1;
				#0 MtempReg[0] = SO;
				#(tHALF/2) if(MbitCounter < 8 && MbitCounter != 0) 
					begin
					//if(MnextState != `holdS)
						MbitCounter = MbitCounter + 1;
					end
				end
			endcase
			end
		`sendingStopStart:
			begin
			seqCounter = lSeq;
			end
		endcase
    end
	end
//----------------------------------------------------------------------------------------

//always @(negedge SCK && (mode==2'b01 || mode==2'b10) ) 
always @(negedge SCK) 
	begin
	if(mode==2'b01 || mode==2'b10)
     begin
	case(Mstate)
		`receivingData:		// 01 & 10
			begin
			#(tHALF/2) case(MbitCounter)
			1:
				begin
				MtempReg[7:0] = 8'b00000000;
				#0 MtempReg[0] = SO;
				#tHALF if(MbitCounter < 8 && MbitCounter != 0) 
					begin
					MbitCounter = MbitCounter + 1;
					end
				end
			8:
				begin
				MtempReg = MtempReg << 1;
				MtempReg[0] = SO;
				seqCounter = seqCounter - 1;
				if(seqCounter == 0)
					begin
					bufReadData[recDataCounter] = MtempReg[7:0];
					recDataCounter = recDataCounter + 1;
					crtMemAddr = crtMemAddr + 1;	//(inc.adr.in pag)
					realCrtMemAddr = crtMemAddr;
					if(recDataCounter == lSeq)
						displayRecData;
					MnextState = `sendingStopStart;
					semForever = 1;
					/// #(tHALF/2) MbitCounter = 0;
					#(tHALF/4) MbitCounter = 0;
					//$display("            MASTER: rec. data %d: 		%h",lSeq-seqCounter,MtempReg[7:0]);
					if(Minstruction == RDSR)
						begin
						manStReg[7:0] = MtempReg[7:0];
						//displayStatusRegSense;
						end
					/// #tHALF if(MbitCounter < 8 && MbitCounter != 0) 
					#(tHALF/4) if(MbitCounter < 8 && MbitCounter != 0) 
						MbitCounter = MbitCounter + 1;
					end
				else
					begin
					bufReadData[recDataCounter] = MtempReg[7:0];
					recDataCounter = recDataCounter + 1;
					crtMemAddr = crtMemAddr + 1;	//(inc.adr.in pag)
					realCrtMemAddr = crtMemAddr;
					if(recDataCounter == lSeq)
						displayRecData;
					MnextState = `receivingData;
					//$display("            MASTER: rec. data %d: 		%h",lSeq-seqCounter,MtempReg[7:0]);
					#(tHALF/2) MbitCounter = 1;
					end
				end
			// cases 2,3,4,5,6,7
			default:
				begin
				MtempReg = MtempReg << 1;
				#0 MtempReg[0] = SO;
				#(tHALF/2) if(MbitCounter < 8 && MbitCounter != 0) 
					begin
					MbitCounter = MbitCounter + 1;
					end
				end
			endcase
			end
		`sendingStopStart:
			begin
			seqCounter = lSeq;
			end
		endcase
    end
	end


//-------------------------------------------------------------------------- 00/11
//
//	State machine for SEND type MASTER activity (00/11 modes)
//
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ 01/10 

//------------------------------------------------------------------------------
// always @(negedge SCK && (mode==2'b00 || mode==2'b11) && MnextState != `holdS) 

always @(negedge SCK && 1) 
	begin
	if( ($time == 0) || ((mode==2'b00 || mode==2'b11) && MnextState != `holdS) )
     begin
	prevState = Mstate;
	Mstate = MnextState;
	case(Mstate)
		`sendingInstruction: 		// 00 & 11
			begin
			#(tHALF-tSU)  
			if(MbitCounter < 8) 
				begin
				MbitCounter = MbitCounter + 1;
				end
			if(MbitCounter == 8) 
				begin
				SI = MtempReg[7];
				#(tSU+tH) SI = 1'b0;
				bitA8 = Minstruction[3];
				case(Minstruction)
					WREN:
						begin
						if(MnextState != `holdS)
							begin
							MnextState = `sendingStopStart;
							semForever = 1;
							end
						end

					WRDI:
						begin
						if(MnextState != `holdS)
							begin
							MnextState = `sendingStopStart;
							semForever = 1;
							end
						end

					RDSR:
						begin
						if(MnextState != `holdS)
							begin
							recDataCounter = 0;
							initBufReadData;
							lSeq = 1;
							seqCounter = lSeq;
							MnextState = `receivingData;
							SI= 1'b0;
							#(tHALF-tH) MbitCounter = 1;
							end
						end
					WRSR:
						begin
						if(MnextState != `holdS)
							begin
							lSeq = 1;
							seqCounter = lSeq;
							MnextState = `sendingData;
							end
						end					
					READ:
						begin
						if(MnextState != `holdS)
							begin
							seqCounter = lSeq;
							recDataCounter = 0;
							initBufReadData;
							MnextState = `sendingAdd;
							addrBytesCounter = addrBytes;
							end
						end					
					WRITE:
						begin
						if(MnextState != `holdS)
							begin
							seqCounter = lSeq;
							MnextState = `sendingAdd;
							addrBytesCounter = addrBytes;
							createIncremFillPattern;
							end
						end
					default:
						begin
						MnextState = `sendingStopStart;
						semForever = 1;
						end
				endcase
				end
			else
				begin
				SI = MtempReg[7];
				#(tSU+tH) SI = 1'b0;
				#0 MtempReg = MtempReg << 1;
				end
			end

		`sendingAdd:		// 00 & 11
			begin
			if(MbitCounter==8 && sem == 0)
				begin
				#(tHALF-tSU) MbitCounter=1;
				sem = 1;
				end
			case(MbitCounter)
			1:
				begin
				if(addr > addrMax)
					begin
					$display;
					$display("\t *** WARNING: address specified in Input.v (%h) higher than the maximum one (%h) !",addr,addrMax);
					for(i=15; i >= addrBits; i=i-1)
						addr[i]=0;	
					$display("\t ***          it will be considered as %h !",addr);
					$display;
				end
				if(addrBytesCounter == 2) 
					begin
					MtempReg = addr[15:8];
					$display("            MASTER: sent addr[15:8]:             %h",MtempReg);
					end
				else
					begin
					MtempReg = addr[7:0];
					$display("            MASTER: sent addr[ 7:0]:             %h",MtempReg);
					end
				 if(MnextState != `holdS)
					begin
					SI = MtempReg[7];
					#0 MtempReg = MtempReg << 1;
					end
				#(tSU+tH) SI = 0;
				#(tHALF-tSU-tH) if(MnextState != `holdS)
					begin
					if(prevState != `holdS)
						begin
						// $display("AAAAAAAAA MnextState = %b",MnextState,$time);
						#tHALF MbitCounter = MbitCounter + 1;
						end
					else
						begin
						// $display("BBBBB",$time);
						MbitCounter = MbitCounter + 1;
						end
					SI = MtempReg[7];
					MtempReg = MtempReg << 1;
					#(tSU+tH) SI = 0;
					end
				end
			default:
				begin
				#(tHALF-tSU) SI = MtempReg[7];
				if(MbitCounter < 8) 
					begin
					MbitCounter = MbitCounter + 1;
					end
				#(tSU+tH) SI = 0;
				MtempReg = MtempReg << 1;
				if(MbitCounter==8)
					begin
					sem = 0;
					case(addrBytesCounter)
					2:
						begin
						if(MnextState != `holdS)
							begin
							MnextState = `sendingAdd;
							addrBytesCounter = addrBytesCounter-1;
							#(2*tHALF-tSU-tH) MbitCounter = 1;
							MtempReg = addr[7:0];
							$display("            MASTER: sent addr[ 7:0]:             %h",MtempReg);
							SI = MtempReg[7];
							#0 MtempReg = MtempReg << 1;
							#(tSU+tH) SI = 0;
							#(2*tHALF-tSU-tH)if(MnextState != `holdS)
								begin
								MbitCounter = MbitCounter + 1;
								SI = MtempReg[7];
								MtempReg = MtempReg << 1;
								#(tSU+tH) SI = 0;
								end
							end
						end
					1:
						begin
						if(Minstruction==READ) 
							begin
							if(MnextState != `holdS)
								begin
								MnextState = `receivingData;
								SI= 1'b0;
								#(tHALF-tH) MbitCounter = 1;
								end
							end
						else
							begin
							if(MnextState != `holdS)
								begin
								MnextState = `sendingData;
								SI= 1'b0;
								end
							end
						addrBytesCounter = addrBytesCounter-1;
						end
					endcase
					end
				end
			endcase
			end


		`sendingData:		// 00 & 11
			begin
			if(MbitCounter==8 && sem == 0)
				begin
				#(tHALF-tSU) MbitCounter=1;
				sem = 1;
				end
			case(MbitCounter)
			1:
				begin

				if(lSeq ==1)
					begin
					MtempReg = data[0];
					bufWriteData[0] = data[0];
					end
				else
					begin
					MtempReg = bufWriteData[lSeq-seqCounter];
					end
				SI = MtempReg[7];
				$display("            MASTER: sent data %d: 		%h",lSeq-seqCounter+1,MtempReg);
				MtempReg = MtempReg << 1;
				#(tSU+tH) SI = 0;
				#(tHALF-tSU-tH) if(MnextState != `holdS)
					begin
					if(prevState != `holdS)
						#tHALF MbitCounter = MbitCounter + 1;
					else
						MbitCounter = MbitCounter + 1;
					end
				SI = MtempReg[7];
				MtempReg = MtempReg << 1;
				#(tSU+tH) SI = 0;
				end
			default:
				begin
				#(tHALF-tSU) SI = MtempReg[7];
				if(MbitCounter < 8) 
					MbitCounter = MbitCounter + 1;
				#(tSU+tH) SI = 0;
				MtempReg = MtempReg << 1;
				if(MbitCounter==8)
					begin
					sem = 0;
					seqCounter = seqCounter - 1;
					if(seqCounter == 0)
						begin		
						MbitCounter = 0;
						Mstate = `sendingStopStart;
						MnextState = `sendingStopStart;
						#(tHALF-tH-delta) semForever = 1;
						if(Minstruction != WRSR)
							begin
							seqLength = lSeq;
							displayData;
							end
						end
					else
						begin
						sem=0;
						#(2*tHALF-tSU-tH) MbitCounter = 1;
						MtempReg = bufWriteData[lSeq-seqCounter];
						//$display("            MASTER: sent data %d:           %h",lSeq-seqCounter+1,MtempReg);
						SI = MtempReg[7];
						#0 MtempReg = MtempReg << 1;
						#(tSU+tH) SI = 0;
						#(2*tHALF-tSU-tH) MbitCounter = MbitCounter + 1;
						SI = MtempReg[7];
						MtempReg = MtempReg << 1;
						#(tSU+tH) SI = 0;
						end
					end
				end
			endcase
			end

		`sendingStopStart:	// 00 & 11
			begin
			if(mode == 2'b11 || mode == 2'b10)
				SCK = 1'b1;
			semForever = 1;
			MbitCounter = 0;
			addr = 16'h0000;
			crtMemAddr = 16'h0000;
			realCrtMemAddr = 16'h0000;
			seqCounter = lSeq;
			//
			//	Test next steps
			//
			if(prevInstruction != WRITE && (prevInstruction != RDSR || (prevInstruction == RDSR && manStReg[0] == 1'b0)) )
				begin
				case(TestPhase)
				`include "Input.v"

				default:
				$stop;
				endcase
				TestPhase = TestPhase + 1;
				end
			else
				begin
				if(prevInstruction == WRITE || (prevInstruction == RDSR && manStReg[0] == 1'b1) ) // yet not ready
					begin
					Minstruction = RDSR;
					if(prevInstruction == WRITE )
						$display("......Master reading status until ready after internal writing...");
					end
				end
			//
			prevInstruction = Minstruction;
			manNr = TestPhase-1;
			#0 case(Minstruction)
			WREN:
				begin
				$display(" ");
				$display("******************************************************************");
				$display("%d) WREN  (Enable Write)    Time: ",manNr,$time);
				end
			WRDI:
				begin
				$display(" ");
				$display("******************************************************************");
				$display("%d) WRDI  (Disable Write)    Time: ",manNr,$time);
				end
			RDSR:
				begin
				$display(" ");
				$display("******************************************************************");
				$display("%d) RDSR  (Read Status Register)   Time: ",manNr,$time);
				end
			WRSR:
				begin
				$display(" ");
				$display("******************************************************************");
				$display("%d) WRSR  (Write Status Register)   Time: ",manNr,$time);
				$display("    data: %h",data[0]);
				end
			READ:
				begin
				$display(" ");
				$display("******************************************************************");
				$display("%d) READ  (Read Data from Memory)   Time: ",manNr,$time);
				$display("    address:         %h",addr);
				$display("    sequence length: %d",lSeq);
				end
			WRITE:
				begin
				$display(" ");
				$display("******************************************************************");
				$display("%d) WRITE  (Write Data to Memory)  Time: ",manNr,$time);
				$display("    address:         %h",addr);
				$display("    sequence length: %d",lSeq);
				end
			default:
				begin
				$display(" ");
				$display("******************************************************************");
				$display("%d) WRONG_OPCODE  Time: ",manNr,$time);
				end
			endcase
			//
			if(mode == 2'b00 || mode == 2'b01)
				begin
				seqCounter = lSeq;
				#tCSH CSB = 1;	
				end
			else
				begin
				seqCounter = lSeq;
				//=============
				#tCSH CSB = `B1;	// STOP
				end

			if(mode == 2'b00 || mode == 2'b01)
				begin
				//=================
				#tCS CSB =`B0;		//START
				//	$display("            MASTER: sending START at time ",$time);
				Mstate = `sendingInstruction;
				MnextState = `sendingInstruction;
				#(tCSS-tSU) MbitCounter = 0;
				if(mode == 2'b0)
					begin
					MtempReg = Minstruction;

					`ifdef cat25c04
						if(Minstruction == READ || Minstruction == WRITE)
							MtempReg[3] = addr[8];
					`endif

					`ifdef cat25040
						if(Minstruction == READ || Minstruction == WRITE)
							MtempReg[3] = addr[8];
					`endif


					$display("            MASTER: sent instruction:         %b",MtempReg);
					SI = MtempReg[7];
					#0 MtempReg = MtempReg << 1;
					//if(MbitCounter == 0)
					MbitCounter = MbitCounter+1;
					#tSU SCK = `B1;
					semForever = 0;
			 		->reinitClockForStart;
					#tH SI=0;
					end
				else
					begin
					MtempReg = Minstruction;
					`ifdef cat25c04
						if(Minstruction == READ || Minstruction == WRITE)
							MtempReg[3] = addr[8];
					`endif
					`ifdef cat25040
						if(Minstruction == READ || Minstruction == WRITE)
							MtempReg[3] = addr[8];
					`endif

					$display("            MASTER: sent instruction:         %b",MtempReg);
					SI = MtempReg[7];
					#0 MtempReg = MtempReg << 1;
					MbitCounter = MbitCounter+1;

					#tSU SCK = `B1;
					savState = MnextState;
					MnextState = `idle;
					semForever = 0;
			 		->reinitClockForStart;			
					#(tHALF/2) MnextState = savState;
					end
				//=================
				end
			else			//mode == 2'b11 || mode == 2'b10
				begin
				if(mode == 2'b11)
					begin
					//=================
					#400 SCK = `B1;
					semForever = 0;
			 		->reinitClockForStart;
					//--------------
					#(tHALF/2) CSB =`B0;		//START
					//	$display("            MASTER: sending START at time ",$time);
					#(tHALF/2) Mstate = `sendingInstruction;
					MnextState = `sendingInstruction;
					MbitCounter = 0;
					MtempReg = Minstruction;
					`ifdef cat25c04
						if(Minstruction == READ || Minstruction == WRITE)
							MtempReg[3] = addr[8];
					`endif
					`ifdef cat25040
						if(Minstruction == READ || Minstruction == WRITE)
							MtempReg[3] = addr[8];
					`endif

					$display("            MASTER: sent instruction:         %b",MtempReg);
					SI = MtempReg[7];
					#0 MtempReg = MtempReg << 1;
					#tHALF MbitCounter = MbitCounter+1;
					//================
					end
				else		// mode 10
					begin
					//==============
					#400 SCK = `B1;
					Mstate = `sendingInstruction;
					MnextState = `sendingInstruction;
					MbitCounter = 0;
					MtempReg = Minstruction;
					`ifdef cat25c04
						if(Minstruction == READ || Minstruction == WRITE)
							MtempReg[3] = addr[8];
					`endif
					`ifdef cat25040
						if(Minstruction == READ || Minstruction == WRITE)
							MtempReg[3] = addr[8];
					`endif


					$display("            MASTER: sent instruction:         %b",MtempReg);
					SI = MtempReg[7];
					#0 MtempReg = MtempReg << 1;
					#tHALF MbitCounter = MbitCounter+1;
					semForever = 0;
					->reinitClockForStart;
					//--------------
					#(tHALF/2) CSB =`B0;		//START
					//	$display("            MASTER: sending START at time ",$time);
					//================
					end
				end
			end
	endcase 
    end
	end 

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
always @(posedge SCK) 
	begin
	if(CSB == 1'b0 && (mode==2'b01 || mode==2'b10) && MnextState != `holdS) 
	begin
	prevState = Mstate;
	Mstate = MnextState;
	case(Mstate)
		`sendingInstruction: 		// 01 & 10
			begin
			#(tHALF-tSU) if(MbitCounter < 8) 
				begin
				MbitCounter = MbitCounter + 1;
				end
			if(MbitCounter == 8) 
				begin
				SI = MtempReg[7];
				#(tSU+tH) SI = 1'b0;
				case(Minstruction)
					WREN:
						begin
						if(MnextState != `holdS)
							begin
							MnextState = `sendingStopStart;
							semForever = 1;
							end
						end

					WRDI:
						begin
						if(MnextState != `holdS)
							begin
							MnextState = `sendingStopStart;
							semForever = 1;
							end
						end

					RDSR:
						begin
						if(MnextState != `holdS)
							begin
							recDataCounter = 0;
							initBufReadData;
							lSeq = 1;
							seqCounter = lSeq;
							MnextState = `receivingData;
							SI= 1'b0;
							#(tHALF-tH) MbitCounter = 1;
							end
						end
					WRSR:
						begin
						if(MnextState != `holdS)
							begin
							lSeq = 1;
							seqCounter = lSeq;
							MnextState = `sendingData;
							end
						end					
					READ:
						begin
						if(MnextState != `holdS)
							begin
							seqCounter = lSeq;
							recDataCounter = 0;
							initBufReadData;
							MnextState = `sendingAdd;
							addrBytesCounter = addrBytes;
							end
						end					
					WRITE:
						begin
						if(MnextState != `holdS)
							begin
							seqCounter = lSeq;
							MnextState = `sendingAdd;
							addrBytesCounter = addrBytes;
							createIncremFillPattern;
							end
						end
					default:
						begin
						MnextState = `sendingStopStart;
						semForever = 1;
						end

				endcase
				end
			else
				begin
				SI = MtempReg[7];
				#(tSU+tH) SI = 1'b0;
				//$display("            MASTER: sending command     %b",SI);
				#0 MtempReg = MtempReg << 1;
				end
			end

		`sendingAdd:		// 01 & 10
			begin
			if(MbitCounter==8 && sem == 0)
				begin
				#(tHALF-tSU) MbitCounter=1;
				sem = 1;
				end

			case(MbitCounter)
			1:
				begin
				if(addr > addrMax)
					begin
					$display;
					$display("\t *** WARNING: address specified in Input.v (%h) higher than the maximum one (%h) !",addr,addrMax);
					for(i=15; i >= addrBits; i=i-1)
						addr[i]=0;	
					$display("\t ***          it will be considered as %h !",addr);
					$display;
					end
				if(addrBytesCounter == 2) 
					begin
					MtempReg = addr[15:8];
					$display("            MASTER: sent addr[15:8]:             %h",MtempReg);
					end
				else
					begin
					MtempReg = addr[7:0];
					$display("            MASTER: sent addr[ 7:0]:             %h",MtempReg);
					end
				SI = MtempReg[7];
				#0 MtempReg = MtempReg << 1;
				#(tSU+tH) SI = 0;
///////////
				/// #(2*tHALF-tSU-tH) MbitCounter = MbitCounter + 1;
				#(tHALF-tSU-tH) if(MnextState != `holdS)
					begin
					if(prevState != `holdS)
						begin
						//$display("AAAA",$time);
						#tHALF MbitCounter = MbitCounter + 1;
						end
					else
						begin
						//$display("BBB",$time);
						#(2*tHALF) MbitCounter = MbitCounter + 1;
						end
					end

///////////
				//if(MnextState != `holdS)
				//	begin
					SI = MtempReg[7];
					MtempReg = MtempReg << 1;
					#(tSU+tH) SI = 0;
				//	end
				end
			default:
				begin
				#(tHALF-tSU) SI = MtempReg[7];
				if(MbitCounter < 8) 
					MbitCounter = MbitCounter + 1;
				#(tSU+tH) SI = 0;
				MtempReg = MtempReg << 1;
				if(MbitCounter==8)
					begin
					sem = 0;
					case(addrBytesCounter)
					2:
						begin
						if(MnextState != `holdS)
							begin
							MnextState = `sendingAdd;
							addrBytesCounter = addrBytesCounter-1;
							#(2*tHALF-tSU-tH) MbitCounter = 1;
							MtempReg = addr[7:0];
							$display("            MASTER: sent addr[ 7:0]:             %h",MtempReg);
							SI = MtempReg[7];
							#0 MtempReg = MtempReg << 1;
							#(tSU+tH) SI = 0;
							#(2*tHALF-tSU-tH) if(MnextState != `holdS)
								MbitCounter = MbitCounter + 1;
							SI = MtempReg[7];
							MtempReg = MtempReg << 1;
							#(tSU+tH) SI = 0;
							end
						end
					1:
						begin
						if(Minstruction==READ ) 
							begin
							if(MnextState != `holdS)
								begin
								MnextState = `receivingData;
								SI= 1'b0;
								#(tHALF-tH-delta) MbitCounter = 1;
								end
							end
						else
							begin
							if(MnextState != `holdS)
								begin
								MnextState = `sendingData;
								SI= 1'b0;
								end
							end
						addrBytesCounter = addrBytesCounter-1;
						end
					endcase
					end
				end
			endcase
			end

		`sendingData:		// 01 & 10
			begin
			if(MbitCounter==8 && sem == 0)
				begin
				#(tHALF-tSU) MbitCounter=1;
				sem = 1;
				end
			case(MbitCounter)
			1:
				begin

				if(lSeq ==1)
					begin
					MtempReg = data[0];
					bufWriteData[0] = data[0];
					end
				else
					MtempReg = bufWriteData[lSeq-seqCounter];
				SI = MtempReg[7];
				// $display("            MASTER: sent data %d: 		%h",lSeq-seqCounter+1,MtempReg);
				MtempReg = MtempReg << 1;
				#(tSU+tH) SI = 0;
///////////
///				#(2*tHALF-tSU-tH) MbitCounter = MbitCounter + 1;
				#(tHALF-tSU-tH) if(MnextState != `holdS)
					begin
					if(prevState != `holdS)
						#tHALF MbitCounter = MbitCounter + 1;
					else
						#(2*tHALF) MbitCounter = MbitCounter + 1;
					end
//////////
				SI = MtempReg[7];
				MtempReg = MtempReg << 1;
				#(tSU+tH) SI = 0;
				end
			default:
				begin
				#(tHALF-tSU) SI = MtempReg[7];
				if(MbitCounter < 8) 
					MbitCounter = MbitCounter + 1;
				#(tSU+tH) SI = 0;
				MtempReg = MtempReg << 1;
				if(MbitCounter==8)
					begin
					sem = 0;
					seqCounter = seqCounter - 1;
					if(seqCounter == 0)
						begin		
						semForever = 1;
						MbitCounter = 0;
						Mstate = `sendingStopStart;
						MnextState = `sendingStopStart;
						if(Minstruction != WRSR)
							begin
							seqLength = lSeq;
							displayData;
							end
						#(tHALF-tH-delta) semForever = 1;
						end
					else
						begin
						sem=0;
						#(2*tHALF-tSU-tH) MbitCounter = 1;
						MtempReg = bufWriteData[lSeq-seqCounter];
						//$display("            MASTER: sent data %d:           %h",lSeq-seqCounter+1,MtempReg);
						SI = MtempReg[7];
						#0 MtempReg = MtempReg << 1;
						#(tSU+tH) SI = 0;
						#(2*tHALF-tSU-tH) MbitCounter = MbitCounter + 1;
						SI = MtempReg[7];
						MtempReg = MtempReg << 1;
						#(tSU+tH) SI = 0;
						end
					end
				end
			endcase
			end

		`sendingStopStart:		// 01 & 10
			begin
			if(mode == 2'b11 || mode == 2'b10)
				SCK = 1'b1;
			else
				SCK = 1'b0;
			MbitCounter = 0;

			addr = 16'h0000;
			crtMemAddr = 16'h0000;
			realCrtMemAddr = 16'h0000;

			seqCounter = lSeq;
			//
			//	Test next steps
			//
			if(prevInstruction != WRITE  && (prevInstruction != RDSR || (prevInstruction == RDSR && manStReg[0] == 1'b0)) )
				begin
				case(TestPhase)
				`include "Input.v"
		//		`include "Input.v"
				default:
				$stop;
				endcase
				TestPhase = TestPhase + 1;
				end
			else
				begin
				if(prevInstruction == WRITE  || (prevInstruction == RDSR && manStReg[0] == 1'b1) ) // yet not ready
					begin
					Minstruction = RDSR;
					if(prevInstruction == WRITE )
						$display("......Master reading status until ready after internal writing...");
					end
				end
			//
			prevInstruction = Minstruction;
			manNr = TestPhase-1;
			#0 case(Minstruction)
			WREN:
				begin
				$display(" ");
				$display("******************************************************************");
				$display("%d) WREN  (Enable Write)    Time: ",manNr,$time);
				end
			WRDI:
				begin
				$display(" ");
				$display("******************************************************************");
				$display("%d) WRDI  (Disable Write)    Time: ",manNr,$time);
				end
			RDSR:
				begin
				$display(" ");
				$display("******************************************************************");
				$display("%d) RDSR  (Read Status Register)   Time: ",manNr,$time);
				end
			WRSR:
				begin
				$display(" ");
				$display("******************************************************************");
				$display("%d) WRSR  (Write Status Register)   Time: ",manNr,$time);
				$display("    data: %h",data[0]);
				end
			READ:
				begin
				$display(" ");
				$display("******************************************************************");
				$display("%d) READ  (Read Data from Memory)   Time: ",manNr,$time);
				$display("    address:         %h",addr);
				$display("    sequence length: %d",lSeq);
				end
			WRITE:
				begin
				$display(" ");
				$display("******************************************************************");
				$display("%d) WRITE  (Write Data to Memory)  Time: ",manNr,$time);
				$display("    address:         %h",addr);
				$display("    sequence length: %d",lSeq);
				end
			default:
				begin
				$display(" ");
				$display("******************************************************************");
				$display("%d) WRONG_OPCODE  Time: ",manNr,$time);
				end
			endcase
			//
			if(mode == 2'b01)
				begin
				seqCounter = lSeq;
				// STOP
				#tCSH CSB = 1;	
				end
			else
				begin
				seqCounter = lSeq;
				//=============
				#tCSH CSB = `B1;	// STOP
				end

			if(mode == 2'b01)
				begin
				//=================
				#tCS CSB =`B0;		//START
				//	$display("            MASTER: sending START at time ",$time);
				//
				Mstate = `sendingInstruction;
				MnextState = `sendingInstruction;
				#(tCSS-tSU) MbitCounter = 0;
				MtempReg = Minstruction;

				`ifdef cat25c04
					if(Minstruction == READ || Minstruction == WRITE)
						MtempReg[3] = addr[8];
				`endif
				`ifdef cat25040
					if(Minstruction == READ || Minstruction == WRITE)
						MtempReg[3] = addr[8];
				`endif


				$display("            MASTER: sent instruction:         %b",MtempReg);

				if(mode == 2'b01)
					begin
					SI = MtempReg[7];
					#0 MtempReg = MtempReg << 1;
					end

				MbitCounter = MbitCounter+1;
				//================
				#tSU SCK = `B1;
				semForever = 0;
			 	->reinitClockForStart;
				#tH SI=0;
				//=================
				end
			if(mode == 2'b11 || mode == 2'b10)	//mode == 2'b11 || mode == 2'b10
				begin
				//=================
				#400 SCK = `B1;
				semForever = 0;
			 	->reinitClockForStart;
					//--------------
				#(tHALF/2) CSB =`B0;		//START
				//	$display("            MASTER: sending START at time ",$time);
				//
				#(tHALF/2) Mstate = `sendingInstruction;
				MnextState = `sendingInstruction;
				MbitCounter = 0;
				if(MbitCounter == 0)
					begin
					MtempReg = Minstruction;
					`ifdef cat25c04
						if(Minstruction == READ || Minstruction == WRITE)
							MtempReg[3] = addr[8];
					`endif
					`ifdef cat25040
						if(Minstruction == READ || Minstruction == WRITE)
							MtempReg[3] = addr[8];
					`endif

					$display("            MASTER: sent instruction:         %b",MtempReg);
					SI = MtempReg[7];
					#0 MtempReg = MtempReg << 1;
					end
				MbitCounter = MbitCounter+1;
				//================
				end
			end
	endcase 
   end 
	end
endmodule

