/**********************************************************************************
   Copyright(c) Catalyst Semiconductor, Inc., 2005. All rights reserved.

This code is provided as is without warranty or guarantee of any kind as its accuracy, 
completeness, operability, fitness for particular purpose, or any other warranty.

   File        :  SPIslave.v
   Description :  Verilog model for CAT25XXXX SPI serial EEPROMs
		  (viewd as Verilog model for SPI protocol - slave side)

   Version     :  Rev1.00 
   Release     :  Jan, 2005
**********************************************************************************/

`timescale 1ns/1ns	 

//
//		Slave for SPI protocol
//	00 && 11 modes (input data available on the positive edge of SCK)
//
module SPIslave0011(CSB,HOLDB,SCK,SI,SO,WPB);
	output SO;
	reg SO;
	input SI,SCK,CSB,HOLDB,WPB;
//
//		states
//
`define idle 			"idle"
`define receivingInstruction 	"receivingInstruction"
`define receivingAdd 		"receivingAdd"
`define receivingData		"receivingData"
`define sendingData		"sendingData"
`define hold			"hold"
//
//		instructions
//
`define EnableWrite 		1'b000
`define DisableWrite 		1'b001
`define ReadStatus 		1'b010
`define WriteStatus 		1'b011
`define ReadData 		1'b100
`define WriteData 		1'b101
//
`define lgMemblk	32768		// //--------------------------------------------------------------------
// 	Read chip type
//
`include "MemType.v"
//
//---------------------------------------------------------------------
// 	Read timing
//
// parameter frequency = 2000000; //(Hz)
	`ifdef cat25c01
		`include "Tim_1_2_4_8_16.v"
	`endif
	`ifdef cat25010
		`include "Tim_10_20_40.v"
	`endif
	`ifdef cat25c02
		`include "Tim_1_2_4_8_16.v"
	`endif
	`ifdef cat25020
		`include "Tim_10_20_40.v"
	`endif
	`ifdef cat25c04
		`include "Tim_1_2_4_8_16.v"
	`endif
	`ifdef cat25040
		`include "Tim_10_20_40.v"
	`endif
	`ifdef cat25c08
		`include "Tim_1_2_4_8_16.v"
	`endif
	`ifdef cat25c16
		`include "Tim_1_2_4_8_16.v"
	`endif
	`ifdef cat25c32
		`include "Tim_32_64.v"
	`endif
	`ifdef cat25c64
		`include "Tim_32_64.v"
	`endif
	`ifdef cat25c128
		`include "Tim_128_256.v"
	`endif
	`ifdef cat25c256
		`include "Tim_128_256.v"
	`endif
parameter delta = 10;
//-----------------------------------------------------------------------
integer bitCounter, seqCounter, addrBytesCounter, afterTest;
reg [19:0] i;
reg savSO;
integer tNORM, semForever, sem, semWPEN, interruptWRITE;
integer TestPhase,addrBytes;
integer tOUT;
integer lPage;					// (lgPag = 2**`pgExp)
integer lSeq;					// sequence length 

integer tHALF_MIN;
integer outf;

reg [7:0] WREN, WRDI, RDSR, WRSR, READ, WRITE;
reg [7:0] instruction, prevInstr;		// instruction
reg [8*22:1] state,nextState,savState;     	// State machine controller
reg bitMode, bitA8, memProt;
reg [1:0] mode;	

reg [7:0] realMem[0:32767];			// Real Memory Block (256K bits)
reg [7:0] realStatusRegister;

reg [15:0] addrMax, addrBegFullProt, addrBegHalfProt, addrBegQuarProt, addrBegProt, addrEndProt;

reg [15:0] Saddr;	

reg [7:0] mem[0:32767];			// WORK Memory Block (256K bits)
reg [7:0] statusRegister;
reg [7:0] tempReg,dataReg; 
integer man;
reg [7:0] mansm;
reg [7:0] manStReg;


			time savTim;
time negCSBTim;
time posCSBTim;
			time crtPer;
time negSCKTim;
time posSCKTim;
			time lastStopTim;

//####################### TIMING CHECK ###################################
	specify
        //
        //      timing checks for master send activity (IN)
        //
	// 	clock
	$period(posedge SCK, 1000/fMAX);	// fMAX in MHz, period in ns
	$width(posedge SCK, tWH);
	$width(negedge SCK, tWL);
	//
	// 	data in
        $setuphold(SI, posedge SCK &&& (CSB == 0), tSU, tH);
        //
	endspecify
//######################## TASK ###################################

task displayStatusRegSense;
	begin

manStReg = tempReg;

	$display("-------------------------------");
	$display("Status Register %hH (%b): ",manStReg,manStReg);
	$display("-------------------------------");
	//
	// explain status register
	//
	//	SR[0]=RDY,SR[1]=WEL,SR[2]=BP0,SR[3]=BP1;SR[5]=mode,SR[7]=WPEN
	case (manStReg[0])
		1'b0:
			$display("     Device ready");
		1'b1:
			$display("     Device not ready - write cycle in progress");
	endcase
	case (manStReg[1])
		1'b0:
			$display("     Write disable");
		1'b1:
			$display("     Write enable");
	endcase
	case (manStReg[7])
		1'b0:	if(semWPEN == 1)
				$display("     Write protect enable: 0");
		1'b1:	if(semWPEN == 1)
				$display("     Write protect enable: 1");
	endcase
	case (manStReg[5])
		1'b1:	if(semWPEN == 1)
				$display("     SPI mode:             00/11");
		1'b0:	if(semWPEN == 1)
				$display("     SPI mode:             01/10");
	endcase
	case (manStReg[3:2])
		2'b00:
			$display("     Block protect:        no");
		2'b01:
			$display("     Block protect:        quart memory (%hH - %hH)",addrBegProt,addrEndProt);
		2'b10:
			$display("     Block protect:        half memory (%hH - %hH)",addrBegProt,addrEndProt);
		2'b11:
			$display("     Block protect:        entire memory (%hH - %hH)",addrBegProt,addrEndProt);
	endcase
	if(WPB == 0)
		$display("     WP pin:               %b (hardware protection)",WPB);
	else
		$display("     WP pin:               %b (no hardware protection)",WPB);
	end
endtask

task updateMem;
	begin
	for(i=0; i<`lgMemblk; i=i+1)
		mem[i] = realMem[i];
	end
endtask

task updateRealMem;
	begin
	for(i=0; i<`lgMemblk; i=i+1)
		realMem[i] = mem[i];
	//
	outf = $fopen("data_out.dat");
		for(i=0; i<=addrMax; i=i+1)
			begin
			//if(i%64 == 0)
				$fdisplay(outf,"@0x%h",i);
			$fdisplay(outf,"\t0x%h",mem[i]);
			end
	$fclose(outf);
	//
	end
endtask


//#################################### INITIAL ########################################

initial
	begin

	tHALF_MIN = 1000/(2*fMAX); 
     
	WREN =  8'b00000110;	// Enable Write
	WRDI =  8'b00000100;	// Disable Write
	RDSR =  8'b00000101;	// Read Status Register
	WRSR =  8'b00000001;	// Write Status Register
	READ =  8'b00000011;	// Read Data from Memory
	WRITE = 8'b00000010;	// Write Data to Memory

	$display(" ");
	$display("===========================================================");
	$display("*");
	`ifdef cat25c01
		semWPEN = 1;
		addrBytes = 1;
		`define addrBits 7
		lPage = 16;
		`define pgExp 4
		`define pgExpMinusOne 3
		$display("*             Chip: cat25c01");
	`endif
	`ifdef cat25010
		semWPEN = 0;
		addrBytes = 1;
		`define addrBits 7
		lPage = 16;
		`define pgExp 4
		`define pgExpMinusOne 3
		$display("*             Chip: cat25010");
	`endif
	`ifdef cat25c02
		semWPEN = 1;
		addrBytes = 1;
		`define addrBits 8
		lPage = 16;
		`define pgExp 4
		`define pgExpMinusOne 3
		$display("*             Chip: cat25c02");
	`endif
	`ifdef cat25020
		semWPEN = 0;
		addrBytes = 1;
		`define addrBits 8
		lPage = 16;
		`define pgExp 4
		`define pgExpMinusOne 3
		$display("*             Chip: cat25020");
	`endif
	`ifdef cat25c04
		semWPEN = 1;
		addrBytes = 1;
		`define addrBits 9
		lPage = 16;
		`define pgExp 4
		`define pgExpMinusOne 3
		$display("*             Chip: cat25c04");
	`endif
	`ifdef cat25040
		semWPEN = 0;
		addrBytes = 1;
		`define addrBits 9
		lPage = 16;
		`define pgExp 4
		`define pgExpMinusOne 3
		$display("*             Chip: cat25040");
	`endif
	`ifdef cat25c08
		semWPEN = 1;
		addrBytes = 2;
		`define addrBits 10
		lPage = 32;
		`define pgExp 5
		`define pgExpMinusOne 4
		$display("*             Chip: cat25c08");
	`endif
	`ifdef cat25c16
		semWPEN = 1;
		addrBytes = 2;
		`define addrBits 11
		lPage = 32;
		`define pgExp 5
		`define pgExpMinusOne 4
		$display("*             Chip: cat25c16");
	`endif
	`ifdef cat25c32
		semWPEN = 1;
		addrBytes=2;
		`define addrBits 12
		lPage = 32;
		`define pgExp 5
		`define pgExpMinusOne 4
		$display("*             Chip: cat25c32");
	`endif
	`ifdef cat25c64
		semWPEN = 1;
		addrBytes=2;
		`define addrBits 13
		lPage = 64;
		`define pgExp 6
		`define pgExpMinusOne 5
		$display("*             Chip: cat25c64");
	`endif
	`ifdef cat25c128
		semWPEN = 1;
		addrBytes = 2;
		`define addrBits 14
		lPage = 64;
		`define pgExp 6
		`define pgExpMinusOne 5
		$display("*             Chip: cat25c128");
	`endif
	`ifdef cat25c256
		semWPEN = 1;
		addrBytes = 2;
		`define addrBits 15
		lPage = 64;
		`define pgExp 6
		`define pgExpMinusOne 5
		$display("*             Chip: cat25c256");
	`endif
	//
	//
//
	addrMax = 16'h0000;
	for(i=0; i<`addrBits; i=i+1)
		addrMax[i]=1;	
	//
	addrBegFullProt = 16'h0000;
	 addrBegHalfProt = (addrMax >> 1) + 16'h0001;
	addrBegQuarProt = addrBegHalfProt + (addrMax >> 2) + 16'h0001;
	$display("*");
	$display("* Max. address:                        %h",addrMax);
	$display("* Begin of full protection address:    %h",addrBegFullProt);
	$display("* Begin of half protection address:    %h",addrBegHalfProt);
	$display("* Begin of quarter protection address: %h",addrBegQuarProt );
	mansm = `pgExp;
	$display("* Page length:                %d bytes (2**%d)",lPage,mansm);
	
	`ifdef v1_8__6_0
		$display ("*       Timing for Vcc = 1.8V - 6.0 V");
	`endif
	`ifdef v2_5__6_0
		$display ("*       Timing for Vcc = 2.5V - 6.0 V");
	`endif
	`ifdef v4_5__5_5
		$display ("*       Timing for Vcc = 4.5V - 5.5 V");
	`endif
	man = fMAX;
	$display("*       fMAX = %d MHz",man);
	man = tWH;
	$display("*       tWH  = %d ns",man);
	man = tWL;
	$display("*       fWL  = %d ns",man);
	man = tCS;
	$display("*       tCS  = %d ns",man);
	man = tCSS;
	$display("*       tCSS = %d ns",man);
	man = tCSH;
	$display("*       tCSH = %d ns",man);
	man = tSU;
	$display("*       tSU  = %d ns",man);
	man = tH;
	$display("*       tH   = %d ns",man);
	man = tV;
	$display("*       tV   = %d ns",man);
	man = tHO;
	$display("*       tHO  = %d ns",man);
	man = tWC;
	$display("*       tWC  = %d ns",man);
	$display("*");
	$display("===========================================================");
	$display;
	`ifdef initMemFile
		//
		// memory initialization with data from file
		//
		$readmemh(`initMemFile,mem);
		$readmemh(`initMemFile,realMem);
		$display("Simulated memory array initialization with data from %s...",`initMemFile);
	`elsif
		//
		// memory initialization with ff
		//
		for(i=0; i<`lgMemblk; i=i+1)
			begin
			mem[i] = 8'hff;
			realMem[i] = 8'hff;
			end
			$display("Simulated memory array initialization with 8'hff...");
	`endif
	//$display("Simulated memory array (`lgMemblk bytes) initialization with 8'hff...");
	//
	//
	state = `idle;
	nextState = `idle;
	Saddr = 16'h0000;
	instruction = 8'h00;
	if(semWPEN == 1)
		begin
		realStatusRegister = 8'h60;		// bit5 = 1 (mod 00/11)
		statusRegister = 8'h60;
		end
	else
		begin
		realStatusRegister = 8'hf0;		//
		statusRegister = 8'hf0;		//
		end
	$display("SLAVE: initial Status Register = %hh",realStatusRegister);
	mode=2'b00;
	bitCounter = 0;
	SO = 1'bz;
	interruptWRITE = 1'b0;
	memProt = 1'b0;
end

//############################### ALWAYS ######################################

always @(negedge CSB) 
	negCSBTim = $time;

always @(posedge CSB) 
	posCSBTim = $time;

always @(negedge SCK) 
	negSCKTim = $time;

always @(posedge SCK) 
	posSCKTim = $time;


//---------------------------------------------------------------------------------
/********
always @(negedge WPB)
	begin
	if(realStatusRegister[7] == 1'b1 && instruction == WRSR && CSB == 1'b0) 
		begin
		interruptWRITE = 1'b1;
		$display("======Interrupting WRSR operation by WP going low while CSB is low...");
		end
	end
****/
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
always @(posedge HOLDB)
	begin
	if(state == `hold && $time != 0)
		begin
		if(SCK == 1'b1)
			begin
			$display("SLAVE:  *** WARNING: wrong timing - HOLD up on clock=1 ...	",$time);
			end
		#tLZ SO = savSO;
		state = savState;
		nextState = savState;
		$display("SLAVE:  Leaving HOLD state ...	",$time);
		end
	end

//------------------------------------------------------------------------------------
always @(negedge HOLDB) 	
	begin
	if($time != 0)
		begin
		if(SCK == 1'b1)
			begin
			$display("SLAVE:  *** WARNING: wrong timing - HOLD down on clock=1 ...	",$time);
			end
		if(($time - negSCKTim) < tHD)
			begin
			$display("SLAVE:  *** WARNING: wrong timing - tHD not respected ...	",$time);
			end
		savState = state;
		state = `hold;
		nextState = `hold;
		#tHZ SO = 1'bz;
		$display("SLAVE:  Entering HOLD state ...	",$time);
		end
	end

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
always @(posedge CSB) 
if($time != 0)
begin
   #0 if((posCSBTim - negSCKTim) >= tCSH)
   begin
	if( (SCK == 1'b0 && mode == 2'b00) || (SCK == 1'b1 && mode == 2'b11) ) 
	begin

	if(instruction == WRSR && ( (semWPEN == 0 && WPB == 1) || (semWPEN == 1 && (WPB == 1 && realStatusRegister[7] == 0) )))
		begin
		realStatusRegister = statusRegister;
		$display("..............statusRegister after WRSR = %h ",realStatusRegister);
		end

	if(instruction == WRITE && realStatusRegister[1] == 1'b1 && (memProt  == 1'b0 || (Saddr < addrBegProt || Saddr > addrEndProt)) )
		begin
		if( (semWPEN == 0 && WPB == 1) || semWPEN == 1)
			begin
			$display("_____ Begin of internal writing ____",$time);
			realStatusRegister[0] = 1'b1;
			$display("      status register just beginning internal writing = %h",realStatusRegister);
			$display; 
			updateRealMem;
			#(tWC-tDIS) realStatusRegister[0] = 1'b0;
			$display("_____ End of internal writing ______",$time);
			realStatusRegister[1] = 1'b0; 
			$display("      status register after internal writing = %h",realStatusRegister);
			$display;
			end
		end

	lSeq = 0;
	prevInstr = instruction;
	state = `idle;
	nextState = `idle;
	bitCounter = 0;

	#tDIS SO = 1'bz;
	end
	if(instruction != WREN)
		realStatusRegister[1] = 1'b0; 
        end
else
	begin
		$display("***** SLAVE:  WARNING ");
		$display("              STOP ignored (tCSH - CS Hold Time - not respected: %d < tCSH) ",posCSBTim - negSCKTim,$time);
	end
end
//------------------------------------------------------------------------------------
always @(negedge CSB)
	begin
	#0 if((negCSBTim - posCSBTim) >= tCS)
		begin
		if(SCK == 1'b0)
			begin
			mode = 2'b00;
			nextState = `receivingInstruction;
			end
		else
			begin
			mode = 2'b11;
			#tHALF_MIN nextState = `receivingInstruction;
			end
		end
	else
		begin
		$display("***** SLAVE:  WARNING ");
		$display("              START ignored (tCS - CS High Time - not respected: %d < tCS) ",negCSBTim - posCSBTim,$time);
		end
	end

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// always @(posedge SCK && nextState != `idle && nextState != `hold )

always @(posedge SCK)
	begin
   #0 if(CSB != 1 && nextState != `idle && nextState != `hold)
   begin
	state = nextState;
	case(state)
		`receivingInstruction:
			begin
			bitCounter = bitCounter + 1;
			case(bitCounter)
			1:
				begin
				if((posSCKTim-negCSBTim) >= tCSS)
					begin
					tempReg[7:0] = 8'b00000000;
					#0 tempReg[0] = SI;
					tempReg = tempReg << 1;
					end
				else
					begin
					$display("***** SLAVE:  WARNING");
					$display("              START ignored (tCSS - CS Setup Time - not respected: %d < tCSS) ",posSCKTim-negCSBTim,$time);
					bitCounter = 0;
					end
				statusRegister = realStatusRegister; 
				end
			8:
				begin
				tempReg[0] = SI;
				$display("            SLAVE:  rec. instruction: 	 %b ", tempReg[7:0]);
				instruction = tempReg;

				`ifdef cat25c04
					bitA8 = tempReg[3];	
					instruction[3]=0;
				`endif
				`ifdef cat25040
					bitA8 = tempReg[3];	
					instruction[3]=0;
				`endif

				case(instruction)
					WREN:
						begin
						if(realStatusRegister[0] == 1'b0)
							begin
							state = `idle;
							nextState = `idle;
							realStatusRegister[1] = 1'b1;
							$display("      statusRegister after WREN: %h",realStatusRegister);
							#(2*tHALF_MIN) bitCounter = 0;
							end
						else
							nextState = `idle;
						end
					WRDI:
						begin
						if(realStatusRegister[0] == 1'b0)
							begin
							state = `idle;
							nextState = `idle;
							realStatusRegister[1] = 1'b0;
							$display("      statusRegister after WRDI: %h",realStatusRegister);
							#(2*tHALF_MIN) bitCounter = 0;
							end
						else
							nextState = `idle;
						end
					RDSR:
						begin
						nextState = `sendingData;
						#(tHALF_MIN) bitCounter = 0;
						end
					WRSR:
						begin
						if(realStatusRegister[0] == 1'b0)
							begin
							nextState = `receivingData;
							#(2*tHALF_MIN) bitCounter = 0;
							end
						else
							nextState = `idle;
						end
					READ:
						begin
						if(realStatusRegister[0] == 1'b0)
							begin
							addrBytesCounter = addrBytes;
							nextState = `receivingAdd;
							#(2*tHALF_MIN) bitCounter = 0;
							end
						else
							begin
							nextState = `idle;
							end
						end
					WRITE:
						begin
						if(realStatusRegister[0] == 1'b0)
							begin
							addrBytesCounter = addrBytes;
							nextState = `receivingAdd;
							#(2*tHALF_MIN) bitCounter = 0;
							updateMem;
							end
						else
							nextState = `idle;
						end

					default:
						begin
						$display;
						$display("Slave: WRONG INSTRUCTION CODE (%b) !",tempReg);
						state = `idle;
						nextState = `idle;
						//bitCounter = 0;
						SO = 1'bz;
						//TestPhase = TestPhase + 1;
						end			
				endcase
				end
			default:
				begin
				tempReg[0] = SI;
				tempReg = tempReg << 1;
				end
			endcase
			end
		`receivingAdd:
			begin
			bitCounter = bitCounter + 1;
			case(bitCounter)
			1:
				begin
				tempReg[7:0] = 8'b00000000;
				#0 tempReg[0] = SI;
				tempReg = tempReg << 1;
				end
			8:
				begin
				tempReg[0] = SI;
				case(addrBytesCounter)
				2:
					begin
					nextState = `receivingAdd;
					addrBytesCounter = addrBytesCounter-1;
					#(2*tHALF_MIN) bitCounter = 0;
					Saddr[15:8] = tempReg[7:0];
					$display("            SLAVE:  rec. addr[15:8]:             %h",tempReg[7:0]);
					end
				1:
					begin
					Saddr[7:0] = tempReg[7:0];
					Saddr[15:`addrBits]=0;
					if(addrBytes == 1) 
						begin
						if(`addrBits == 9) 
							begin
							Saddr[8] = bitA8;
							$display("            SLAVE:  calculated rec. addr[ 8:0]:             %h",Saddr[8:0]);
							end
						else
							$display("            SLAVE:  rec. addr[ 7:0]:             %h",Saddr[7:0]);		
						end
					else
						$display("            SLAVE:  rec. addr[ 7:0]:             %h",Saddr[7:0]);
					if(instruction==READ) 
						begin
						nextState = `sendingData;
						#(tHALF_MIN) bitCounter = 0;
						end
					else
						begin
						nextState = `receivingData;
						#(2*tHALF_MIN) bitCounter = 0;
						end
					addrBytesCounter = addrBytesCounter-1;
					end
				endcase

			   end
			default:
				begin
				tempReg[0] = SI;
				tempReg = tempReg << 1;
				end
			endcase
			end
		`receivingData:
			begin
			bitCounter = bitCounter + 1;
			case(bitCounter)
			1:
				begin
				tempReg[7:0] = 8'b00000000;
				#0 tempReg[0] = SI;
				tempReg = tempReg << 1;
				end
			8:
				begin
				tempReg[0] = SI;
				dataReg[7:0] = tempReg[7:0];
				// $display("            SLAVE:  rec. data %d:		%h ", lSeq+1, dataReg[7:0]);
				case(instruction)
					WRSR:
					   begin
					   if(realStatusRegister[1] == 1'b1 && interruptWRITE==1'b0 && (WPB == 1'b1 || realStatusRegister[7] == 1'b0) )
						begin
						statusRegister[3:2] = dataReg[3:2];	// RDY & WEL bits not written by WRSR
						if(semWPEN == 1)
							statusRegister[7] = dataReg[7];
						bitMode = statusRegister[5];
						case(statusRegister[3:2])
							2'b00:
								begin
								//
								//	No protection
								//
								memProt = 1'b0;
								addrBegProt = 16'h0000;
								addrEndProt = 16'h0000;
								//$display("======No protection on memory");
								end
							2'b11:
								begin
								//
								//	Full protection
								//
								memProt = 1'b1;
								addrBegProt = 16'h0000;
								addrEndProt = addrMax;
								//$display("======Full protection on memory");
								end
							2'b10:
								begin
								//
								//	Half protection
								//
								memProt = 1'b1;
								addrBegProt = addrBegHalfProt;
								addrEndProt = addrMax;
								//$display("======Half protection on memory");
								end
							2'b01:
								begin
								//
								//	Quarter protection
								//
								memProt = 1'b1;
								addrBegProt = addrBegQuarProt;
								addrEndProt = addrMax;
								//$display("======Quarter protection on memory");
								end
						endcase
						end
						state =  `idle;
						nextState = `idle;
						statusRegister[1] = 1'b0; 
						#(2*tHALF_MIN) bitCounter = 0;
						interruptWRITE = 1'b0;
					   end
					WRITE:
						begin
						if(realStatusRegister[1] == 1'b1 && (memProt == 1'b0 || (Saddr < addrBegProt || Saddr > addrEndProt)))
							begin
							mem[Saddr] = dataReg[7:0];
							Saddr[`pgExpMinusOne:0] = Saddr[`pgExpMinusOne:0] + 1;//(inc.adr.in pag)
							lSeq = lSeq +1;
							end
						else
							$display("Write not enabled...");
						nextState = `receivingData;
						#(2*tHALF_MIN) bitCounter = 0;
						end	
				endcase // instruction
				end
			default:
				begin
				tempReg[0] = SI;
				tempReg = tempReg << 1;
				end
			endcase
			end
	endcase
    end
	end

//----------------------------------------------------------------------------------------
//always @(negedge SCK && nextState != `idle && nextState != `hold)
always @(negedge SCK)
	begin
	#0 if(CSB != 1 && nextState != `idle && nextState != `hold)
    begin
	state = nextState;
	case(state)
		`sendingData:
			begin
			bitCounter = bitCounter+1;
			case(bitCounter)
				1:
					begin
					#((tV-tHO)/2) case(instruction)
						RDSR:
							begin
							tempReg[7:0] = realStatusRegister[7:0];
							if(state != `hold)
								savSO = tempReg[7];
							if(semWPEN == 0)
								tempReg[7:4] = 4'b1111;
							displayStatusRegSense;
							end
						READ:
							begin
							tempReg[7:0] = mem[Saddr];
							if(state != `hold)
								savSO = tempReg[7];
							if(addrBytes == 2)
								begin
								//$display("      mem[%h] = %h", Saddr, mem[Saddr]);
								Saddr = Saddr + 1;
								end
							else
								begin
								if(`addrBits == 9)
									begin
									//$display("      mem[%h] = %h", Saddr[8:0], mem[Saddr[8:0]]);
									Saddr[8:0] = Saddr[8:0]+1;
									end
								else
									begin
									//$display("      mem[%h] = %h", Saddr[7:0], mem[Saddr[7:0]]);	
									Saddr[7:0] = Saddr[7:0]+1;
									end
								end
							end
						endcase
						lSeq = lSeq + 1;
						SO = tempReg[7];
						#0 tempReg = tempReg << 1;
					end
				8:
					begin
					if(state != `hold)
						savSO = tempReg[7];
					#((tV-tHO)/2) SO = tempReg[7];
					#0 tempReg = tempReg << 1;
					#tV bitCounter = 0;

					case(instruction)
						RDSR:
							begin
							lSeq = 0;
							state = `idle;
							nextState = `idle;
							end
						READ:
							begin
							nextState = `sendingData;
							end
					endcase
					end
				default:
					begin
					if(state != `hold)
						savSO = tempReg[7];
					#((tV-tHO)/2) SO = tempReg[7];
					#0 tempReg = tempReg << 1;
					end
			endcase
			end
		endcase
    end
	end

endmodule

